// Provides a definition of the basic datatype required for lists of string
package main

import (
	"fmt"
)

type Node struct {
	Value string
	Next  *Node // This is a recursive datatype
}

// Given the pointer to the first node of  a list, returns the number of elements
func (first *Node) Length() int {
	var curs *Node
	count := 0
	for curs = first; curs != nil; curs = curs.Next {
		count++
	}
	return count
}

// Given the pointer to the first node of  a list, and a new string,
// adds the latter as the FIRST element of the list. Returns the
// pointer to the new first element.
func (first *Node) AddFront(x string) (newFirst *Node) {
	newFirst = new(Node)
	newFirst.Value = x
	newFirst.Next = first
	return
}

// Given the pointer to the first node of  a list, and a new string,
// adds the latter as the LAST element of the list. Returns the
// pointer to the new first element.
func (first *Node) AddLast(x string) (newFirst *Node) {
	var curs *Node

	newNode := new(Node)
	newNode.Value = x

	if first == nil {
		newFirst = newNode
		return
	}
	// The list is non-empty
	newFirst = first
	for curs = first; curs.Next != nil; curs = curs.Next {
	}
	// curs is now pointing to the last node of the list
	curs.Next = newNode
	return
}

// Given the pointer to the first node of  a list, and a new string,
// adds the latter in the correct position to maintain the list in (lexicographic)
// order. Returns the pointer to the new first element.
func (first *Node) AddInOrder(x string) (newFirst *Node) {
	var curs, prev *Node

	newNode := new(Node)
	newNode.Value = x

	if first == nil {
		newFirst = newNode
		return
	}
	// The list is not empty
	// Exit condition: curs == nil || curs.Value > x
	prev = nil
	for curs = first; curs != nil && curs.Value <= x; curs = curs.Next {
		prev = curs
	}
	if prev == nil { // Inserting as first element
		newNode.Next = first // This is the same as curs, in fact
		newFirst = newNode
		return
	}
	// We put the newNode between prev and curs
	prev.Next = newNode
	newNode.Next = curs
	newFirst = first
	return
}

// Given the pointer to the first node of  a list, prints the list
func (first *Node) Print() {
	var curs *Node
	for curs = first; curs != nil; curs = curs.Next {
		if curs.Next == nil {
			fmt.Printf("%s\n", curs.Value)
		} else {
			fmt.Printf("%s -> ", curs.Value)
		}
	}
}

func main() {
	var first *Node
	first = first.AddFront("pippo").AddFront("pluto").AddLast("cina")
	first.Print()
	fmt.Printf("Lunghezza della lista: %d\n", first.Length())
}
